//===============================================================================
// FILENAME: Oscilloscope.c
//
// DESCRIPTION: This file contains specific routines for
//				 oscilloscope implementation
// NOTE: 
// 
// (c) Entry_AR1755, 2005
//     Philips 2005 ARM Design Contest
//===============================================================================

#include <LPC213x.H> 
#define XMAX        64    //bytes per line (horizontalal resolution)
#define DATALENGHT  320*2 //lenght of sampled data
#define ARRAYLENGHT 256*4 //lenght of data array for saving in FLASH


extern unsigned int VideoArray[];
extern unsigned short ShiftMask;
extern unsigned int  GUI_PrintBigInteger(unsigned int, unsigned int, char, char);
extern void GUI_PrintStr(unsigned int, unsigned int, char*);
extern void GUI_DrawGrid_1(unsigned int);
extern void GUI_ShiftPlotArea();

unsigned char Menu=0;        //Menu level 0--normal operating, 1--"Store", 2--"XXX"
unsigned int cntD_cur=0;     //current pointer to data
unsigned int cntD_old=0;     //previous pointer to data
short cntWait=0;             //counter of frames to wait triggering



unsigned int GetVoltages_ShiftLevel=0;
unsigned int GetVoltages_TrigLevel=0;
unsigned int DataArray[ARRAYLENGHT/4];//data  Array


//================================= ADC0 ISR ====================================
  unsigned int cntD[7]={0,0,0xFFFF,0x0000,10,1,1};	 //multipurpose structure
                 //[0] +0  Current pointer to data (for collecting)
				 //[1] +4  its maximum value
				 //[2] +8  min sample value
				 //[3] +12 max sample value
				 //[4] +16 counter
				 //[5] +20 max val of counter
				 //[6] +24 counter for horizontal ofset
  unsigned int Level=128|0x80000000;	 //Trigger level and flags
//===============================================================================
void ADC0 (void) __irq {
__asm {
  	   LDR   R1,=0xE0034005 //=0xE0034004 -- base address  of AD0DR (ADC data register)
       LDRB  R1,[R1]	    //read 8-bit data from ADC	(bits 8...15)
LDR  R3,=cntD

LDR  R2,=Level
LDR  R0,[R2]	        //load Trigger Level (and flags in bits 31,30)
TST  R0,#0x80000000
BNE  collect		   //if bit31 = 1

TST  R0,#0x40000000
BNE  stady2		       //if bit30 = 1


stady1://wait a lower data	(with de-boincing) 
CMP  R0,R1			    //Trigger Level flag - ADC data =?
BICLS R0,R0,#0x38000000 //if  unsigned lower or same -- clear bits
ADDHI R0,R0,#0x08000000 //else increment (to set in future the bit30) flag for stady 2
STR  R0,[R2]		    //store new flag
B    adc_end

stady2://wait a higher data
AND  R0,R0,#0xFF	   //clear unwanted bits
CMP  R0,R1			   //Trigger Levelflag - ADC data =?
BCS  adc_end		   //if  unsigned higher or same
ORR  R0,R0,#0x80000000 //else set flag  for collect mode
STR  R0,[R2]		   //store new flag
B    adc_end


collect:
	   LDR    R9,[R3,#8]  //load min value
	   LDR    R10,[R3,#12] //load max value

	   //process N samples (to obtain required Horizontal Scale)
	   LDR    R0,[R3,#16]	 //load Counter
       SUBS   R0,R0,#1		 //Counter-1
	   LDREQ  R0,[R3,#20]    //if=0 load Max val of Counter
	   STR    R0,[R3,#16]	 //store Counter
	   BEQ    store			 //if Max val of Counter

	   //choose new max and new min values
	   CMP   R10,R1	    //max?
       STRLO R1,[R3,#12]//if R10<R1
	   CMP   R9, R1	    //min?
       STRHI R1,[R3,#8]	//if R9>R1
	   B     adc_end
store:
	   //implementation of horizontal ofset
	   LDR    R0,[R3,#24]   //counter 2
       SUBS   R0,R0,#1
       STRNE  R0,[R3,#24]   //if C=0 store counter 2 (0 is min value)
	   BNE	  adc_end		//and no process current sample

	   //store current sample
 	   LDR    R0,[R3,#0]    //current pointer -> R0
       STRB   R9,[R0],#1    //store min value
       STRB   R10,[R0],#1   //store max value

	   //current sample as new initial value for Max and Min
       STR    R1,[R3,#8]    //store as min value
       STR    R1,[R3,#12]   //store as max value
       
 	   //increment or not pointer (compare with max)
       LDR    R1,[R3,#4]
	   CMP	  R0,R1			//<= max?
       STRLS  R0,[R3,#0]	//if <= store new pointer
	   BLS    adc_end
	   
	   //else stop ADC
	   LDR    R9,=0xE0034000
	   LDR    R0,[R9]
	   BIC    R0,R0,#0x00200000
	   STR    R0,[R9]
adc_end:
      }//asm
VICVectAddr = 0;      //Acknowledge Interrupt
}

//===============================================================================
// FUNCTION NAME: ADC_Start()
//
// DESCRIPTION:
//            Setup and starts ADCs for data collecting
//
// ARGUMENTS: none
// RETURNS:	  nothing
// SIDE EFFECTS: 
//===============================================================================
void ADC_Start (void){

cntD[1]=(unsigned int)DataArray+(DATALENGHT-2);	 //max value of data pointer
cntD[0]=cntD[1]; //current data pointer = max value	to start collection
cntD_old=cntD[0];

//-------------------------- setup the ADC 1 -----------------------------
AD1CR   = 0x00200000|0x1300 ;       //Setup  ADC1: 10-bit @ 60MHz/0x13+1 (20) = 3 MHz /11= 272.7 ksamples/s
PINSEL0|= 0xCF333000;               //set AIN8,9,10,11,12,13,15 as ADC input pins
PINSEL1|= 0x1000;

//------ set the interrupts in VIC
VICVectAddr2 = (unsigned long)ADC0; // Set Interrupt Vector in 2
VICVectCntl2 = 0x20 | 18;           // Use it for ADC0 Interrupt (8)
VICIntEnable = 0x00040000;          // Enable ADC Interrupt

//-------------------------- start the ADC 0 -----------------------------
PINSEL0|= 0x00000300;               //set AIN6 as ADC pin
AD0CR   = 0x00010040|0x2100 ;       //Setup ADC0: 10-bit @ 60MHz/0x21+1 (34) = 1,76 MHz /11= 160.4 ksamples/s

}


//===============================================================================
// FUNCTION NAME: GetVoltages_CalculateAddr ()
//
// DESCRIPTION:
//            Calculates  address for function GetVoltages_ShowTrigLevel
//
// ARGUMENTS: [GetVoltages_TrigLevel] -- variable, Trigger Level, 10 bit
//
// RETURNS:	  absolute address 
// SIDE EFFECTS: 
//===============================================================================
unsigned int GetVoltages_CalculateAddr(void)
{
return((unsigned int)VideoArray+(3+64*176-(GetVoltages_TrigLevel/8+GetVoltages_TrigLevel/32)*64));
}

//===============================================================================
// FUNCTION NAME: GetVoltages_ShowTrigLevel (addr)
//
// DESCRIPTION:
//            Saves the background and draws the Trigger Level's sprite
//
// ARGUMENTS: addr -- absolute address
//
// RETURNS:	  Nothing
// SIDE EFFECTS: 
//===============================================================================
  const unsigned char ShowTrigLevel_Sprite[7]={0xC1, 0xF0, 0xFC, 0xFF, 0xFC, 0xF0, 0xC1};
  unsigned char  ShowTrigLevel_Temp[7];
//===============================================================================
void GetVoltages_ShowTrigLevel(unsigned int addr) __arm 
{//R0=addr
__asm{
  LDR R1,=ShowTrigLevel_Temp
  LDR R2,=ShowTrigLevel_Sprite
  EOR R0,R0,#0x1   //swap the odd and even addresses

  MOV R3,#7 //counter
loop0:
  LDRB R4,[R0]
  STRB R4,[R1],#1
  LDRB R4,[R2],#1
  STRB R4,[R0],#XMAX
  SUBS R3,R3,#1
  BNE loop0
}//asm
}

//===============================================================================
// FUNCTION NAME: GetVoltages_RestoreTrigLevel (addr)
//
// DESCRIPTION:
//            Restores the background
//
// ARGUMENTS: addr -- absolute address
//
// RETURNS:	  Nothing
// SIDE EFFECTS: 
//===============================================================================
void GetVoltages_RestoreTrigLevel(unsigned int addr) __arm 
{//R0=addr
__asm{
  LDR R1,=ShowTrigLevel_Temp
  EOR R0,R0,#0x1   //swap the odd and even addresses

  MOV R3,#7 //counter
loop0:
  LDRB R2,[R1],#1
  STRB R2,[R0],#XMAX
  SUBS R3,R3,#1
  BNE loop0
}//asm
}

//===============================================================================
// FUNCTION NAME: GetVoltages()
//
// DESCRIPTION:
//            Get all voltages and show 5 of them in fixed screen positions
//			  Save 2 of them in variables GetVoltages_ShiftLevel and GetVoltages_TrigLevel
// ARGUMENTS: none
//
// RETURNS:   nothing
// SIDE EFFECTS:
//===============================================================================
  unsigned char GetVoltages_number=0;
  const unsigned char GetVoltages_mask[5]={0x02,0x04,0x08,0x10,0x20};//voltage sources for  1-st, 2-nd, 3-rd, 4-th, 5-th inputs as a masks for ADC CR
//===============================================================================
void GetVoltages(void)
{int val,zzz;
//next number
GetVoltages_number++;

if (GetVoltages_number>24) GetVoltages_number=0;


//start new conversion and extract data 
AD1CR |= 0x01000000|GetVoltages_mask[GetVoltages_number]; //start ADC1 conversion with new SEL field
do {val=AD1DR;} while((val&0x80000000)==0);               //read Data Register and wait for end of AD Conversion
AD1CR &=~0x010000FF;                                      //Stop AD Conversion and clear SEL field
//normalize
val=(val>>6)&0x3FF;
val=(val*330)>>10;

//print on the screen
zzz=(unsigned int)VideoArray+(4+2+64*210)+GetVoltages_number*12; //calculate the abs addr
if (GetVoltages_number<5)  GUI_PrintBigInteger(zzz,val,3,2);	 //print 3 digits and decimal point


AD1CR |= 0x01000000|0x01;                                 //start ADC1 conversion with new SEL field
do {val=AD1DR;} while((val&0x80000000)==0);               //read Data Register and wait for end of AD Conversion
AD1CR &=~0x010000FF;                                      //Stop AD Conversion and clear SEL field
//normalize
val=(val>>8)&0xFF;
GetVoltages_ShiftLevel=(GetVoltages_ShiftLevel+val)>>1;   //new value of level for hor. ofset (filterd)

AD1CR |= 0x01000000|0x80;                                 //start ADC1 conversion with new SEL field
do {val=AD1DR;} while((val&0x80000000)==0);               //read Data Register and wait for end of AD Conversion
AD1CR &=~0x010000FF;                                      //Stop AD Conversion and clear SEL field
//normalize	 and clear unwanted bits
val=(val>>6)&0x3FC;
GetVoltages_RestoreTrigLevel(GetVoltages_CalculateAddr());//restore area
GetVoltages_TrigLevel=(GetVoltages_TrigLevel+val)>>1;     //new value of Trigger Level for showing  (filtered)
GetVoltages_ShowTrigLevel(GetVoltages_CalculateAddr());	  //draw Trigger Level sprite
Level=(Level&0xFFFFFF00)|(GetVoltages_TrigLevel>>2);	  //new value of of Trigger Level for synchronization
}
//===============================================================================
// FUNCTION NAME: GetKey()
//
// DESCRIPTION:
//            Get a current state of keys using debouncing algorithm
//
// ARGUMENTS: none
//
// RETURNS:   0 if not keypressed, else returns the bits of pressed keys:
//			  bit0=1 if S2 is pressed,
//            bit1=1 if S3 is pressed,
//			  bit2=1 if S4 is pressed,
//			  bit3=1 if S5 is pressed,
//            bit7=1 is once when a key is pressed (the flag against typematic)
// SIDE EFFECTS:
//===============================================================================
  const int GetKey_mask[4]={0x010000,0x020000,0x040000,0x100000}; //masks for buttons S2,S3,S4,S5
  unsigned char GetKey_j=0;	  //counter of non-keypressed cycles
  unsigned char GetKey_result=0;
//===============================================================================
unsigned char GetKey(void)
{int port;  char i;
port=IOPIN0;
if(GetKey_result&0x80) GetKey_result&=~0x80;//clear flag if it has been set
for	(i=0;i<4;i++)
	{if ((port&GetKey_mask[i])==0) {if(GetKey_result==0) GetKey_result|=0x80;   //set flag
	                                GetKey_j=5; //MAX value	for debouncing
	                                GetKey_result|=(1<<i);
								   }//if
	}//for i
if(GetKey_j>0) GetKey_j--; else GetKey_result=0;

   //!!for debug P1.16..23 defined as Outputs
       IODIR1  = 0x00FF0000; 
       if (GetKey_result==0) IOCLR1 = 0x00FF0000;
       if ((GetKey_result&0x80)==0) IOCLR1 = 0x00800000;
       if (GetKey_result&0x0F) IOSET1 = (GetKey_result<<16)&0x00FF0000;


return(GetKey_result);
}
//===============================================================================
// FUNCTION NAME: DrawPlot(next, addr_start, addr_end)
//
// DESCRIPTION:
//            Clears the plot area and visualizes the data
//
// ARGUMENTS: next       -- draw mode 0/1  (0 to begin a plot, 1 to continue drawing)
//			  addr_start -- pointer to data
//			  addr_end   -- end value of pointer
//
// RETURNS:   nothing
//
// SIDE EFFECTS:
//===============================================================================
  unsigned short mask; 	      //mask to draw the point
  unsigned short ofs;		  //ofset in line to draw the point
  char first;				  //flag that value is first in line

  unsigned short valmin;
  unsigned short valmax;
  unsigned short valmin_old;
  unsigned short valmax_old;
  unsigned short val1;
  unsigned short val2;
  unsigned short * Addr1;      //bottom address in VideoArray where the point will be displayed
  unsigned short * Addr2;      //top address in VideoArray where the point will be displayed
//===============================================================================
void DrawPlot(unsigned char next, unsigned char *cntD0, unsigned char *cntD1) __arm
{
if(next==0) {first=0; ofs=0; mask=0x8000;}	//set initial values
if(cntD0>cntD1) return;

for(cntD0;cntD0<=cntD1;) {

if(mask==0x8000) GUI_DrawGrid_1(ofs); //clear vertical area and draw grid there
valmin= *cntD0++;       //current byte
valmax= *cntD0++;       //current byte

//-------- draw a vertical piece ---------
if (first==0) {valmax_old=valmax; valmin_old=valmin; first=1;}; //if first point in line

if(valmin>valmax_old)	 val1=valmax_old;	else val1=valmin;
if(valmax<valmin_old)	 val2=valmin_old;	else val2=valmax;
valmin_old=valmin;
valmax_old=valmax;

if(valmin>valmax) {val1=valmin; val2=valmax; first=0;} //case of unproper data -- don't draw

Addr1=(unsigned short *)VideoArray+ofs+(2+180*32-(val2/2+val2/8)*32); //also adjust a scale
Addr2=(unsigned short *)VideoArray+ofs+(2+180*32-(val1/2+val1/8)*32); //also adjust a scale
//draw vert line
//while (Addr1<=Addr2) {(*Addr1)|=mask; Addr1+=32;}
__asm{
  LDR R1,=Addr1
  LDR R1,[R1]
  LDR R2,=Addr2
  LDR R2,[R2]
  LDR R3,=mask
  LDRH R3,[R3]
  CMP R1,R2
  BHI no_loop
loop:
  LDRH R0,[R1]
  ORR R0,R0,R3
  STRH R0,[R1],#XMAX
  CMP R1,R2
  BLS loop
no_loop:
  }//asm

mask/=2; if(mask==0) {mask=0x8000; ofs++; if(ofs>=20) {ofs=0; first=0;}} //next values of mask and ofs
}//for cntD0

}
//===============================================================================
// FUNCTION NAME: NewHScale(direction)
//
// DESCRIPTION:
//            Set a new horizontal scale
//
// ARGUMENTS: direction  = 1 for incrment or =255 for decrement
//						 = 0 for redraw	only
//
// RETURNS:   nothing
//
// SIDE EFFECTS:
//===============================================================================
  const short NewHScale_value[20]     ={ 1, 2, 5, 1, 2, 5, 10, 20, 50, 100, 200, 500,   1,   2,    5,   10,   20,    50,   100,   200}; //values for showing
  const char  NewHScale_dim1[20]      ={ 2, 2, 2, 1, 1, 1,  2,  2,  2,   3,   3,   3,   1,   1,    1,    2,    2,     2,     3,     3}; //total lenght for showing
  const char  NewHScale_dim2[20]      ={ 1, 1, 1, 0, 0, 0,  0,  0,  0,   0,   0,   0,   0,   0,    0,    0,    0,     0,     0,     0}; //position for decimal point
  const short NewHScale_ADCdevider[20]={33,33,33,33,33,33, 33, 33, 33,  33,  33,  33, 169, 169,  169,  169,  169,   169,   169,  169,}; //devider value for ADC
  const int   NewHScale_N[20]         ={ 1, 2, 5,10,20,50,100,200,500,1000,2000,5000,2000,4000,10000,20000,40000,100000,200000,400000}; //factor N
  unsigned char HScale_index=0;
  unsigned short *ptrD;
//===============================================================================
void NewHScale(unsigned char direction) 
{

if((direction==1)&(HScale_index<(20-1))) HScale_index++;
if((direction==255)&(HScale_index>0))  HScale_index--;

GUI_PrintBigInteger((unsigned int)VideoArray+(48+XMAX*140), 0, 3, 255);	//clear
if(HScale_index<12) GUI_PrintStr(15,47, "\5 \4      mS/div\5 \4");
			  else  GUI_PrintStr(15,47, "\5 \4       S/div\5 \4");
GUI_PrintBigInteger((unsigned int)VideoArray+((6-2*NewHScale_dim1[HScale_index])+48+XMAX*140), NewHScale_value[HScale_index], NewHScale_dim1[HScale_index], NewHScale_dim2[HScale_index]);
cntD[4]=10;							//counter for ADC ISR
cntD[5]=NewHScale_N[HScale_index];  //max val of counter for ADC ISR

if(direction==0) return; //without restarting of collection 

//else restart collecting
cntD[0]=cntD[1]; Level|=0x80000000;//stop collecting		
if((AD0CR&0x200000)!=0)	{};//wait
AD0CR=AD0CR&0xFFFF00FF|(NewHScale_ADCdevider[HScale_index]<<8);//new ADC devider value

for(ptrD=(unsigned short *)DataArray;ptrD<=(unsigned short *)cntD[1];ptrD++) *ptrD=0xFF;//clear data in array
DrawPlot(0,(unsigned char *)DataArray,(unsigned char *)cntD[1]);//draw null plot								   
cntD_old=(unsigned int)DataArray; cntD_cur=cntD_old;
ShiftMask=0x0001;
}
//===============================================================================
// FUNCTION NAME: FLASH_Save(index)
//
// DESCRIPTION:
//            Saves the block of data to FLASH
//
// ARGUMENTS: index -- number of data block 0...10
//
// RETURNS:   nothing
//
// SIDE EFFECTS:
//===============================================================================
  #define FLASH_BASE 0x00008000//base address in FLASH where the data is stored
  #define FLASH_SECTOR 8;      //first sector of FLASH where the data is stored (32K, 0x00008000...0x00008FFFF)
  #define FLASH_NUMBERS 10;	   //total of data blocks
  typedef void (*IAP)(unsigned int[],unsigned int[]);
  IAP iap_call=(IAP)0x7FFFFFF1;
//===============================================================================
void FLASH_Save(unsigned char index) __arm 
{
unsigned command[5];
unsigned result[5];

unsigned short i,ind;//for seeking
unsigned short n=0;	 //number of last used area in FLASH sector (is calculated)

VICIntEnClr=0xFFFFFFFF;        //disable all interrupts
VICIntEnClr=0xFFFFFFFF;        //disable all interrupts

//seek the first unused area in FLASH sector, returns n -- number
for(i=0;i<=31;i++) {ind=*(unsigned short*)(((unsigned)index-1)*0x00008000+i*ARRAYLENGHT+FLASH_BASE+ARRAYLENGHT-2);
                    if(ind!=index) {if((ind==0)|(ind==0xFFFF)) n=i; else n=0;  break;};
				   }

//!!for debug
//GUI_PrintBigInteger((unsigned int)VideoArray+(48+XMAX*50), 0, 3, 255);
//GUI_PrintBigInteger((unsigned int)VideoArray+(48+XMAX*50), n, 3, 0);
//GUI_PrintBigInteger((unsigned int)VideoArray+(48+XMAX*90), 0, 3, 255);
//GUI_PrintBigInteger((unsigned int)VideoArray+(48+XMAX*90), i, 3, 0);

//Specify header for saving
*((unsigned short*)((unsigned int)DataArray+ARRAYLENGHT-2))=index;
*((unsigned short*)((unsigned int)DataArray+ARRAYLENGHT-4))=HScale_index;
*((unsigned short*)((unsigned int)DataArray+ARRAYLENGHT-6))=500; //is fixed in this version
*((unsigned short*)((unsigned int)DataArray+ARRAYLENGHT-8))=DATALENGHT;
*((unsigned int*)  ((unsigned int)DataArray+ARRAYLENGHT-12))=*(unsigned int*)(((unsigned)index-1)*0x00008000+ARRAYLENGHT-12)+1;//erase counter

if(n==0){//all data areas are already used 
		 //in this case clear FLASH sector

          command[0]=50;		//"Prepare sectors"
          command[1]=(unsigned)index-1+FLASH_SECTOR;
          command[2]=command[1];
          iap_call(command,result);

          command[0]=52;	    //"Erase sectors"
          command[1]=(unsigned)index-1+FLASH_SECTOR;
          command[2]=command[1];
          command[3]=60000;		//Cclk == 60Mhz
          iap_call(command,result);
        }//if

//now, n contains the number of unused area in FLASH sector
command[0]=50;			//"Prepare sectors"
command[1]=(unsigned)index-1+FLASH_SECTOR;
command[2]=command[1];
iap_call(command,result);
//Write data
command[0]=51;		    //"Write"
command[1]=((unsigned)index-1)*0x00008000+n*ARRAYLENGHT+FLASH_BASE;   //addr in flash memory;
command[2]=(unsigned int)DataArray;   //from RAM
command[3]=ARRAYLENGHT;
command[4]=60000;	   //Cclk == 60Mhz
iap_call(command,result);

VICIntEnable=0x00000800 | 0x00000100; //Enable interrupts from PWM+SSP, from ADC will be enabled in FIQ ISR

}

//===============================================================================
// FUNCTION NAME: FLASH_Restore(index)
//
// DESCRIPTION:
//            Show the saved graph from FLASH
//
// ARGUMENTS: index -- number of data block 0...10
//
// RETURNS:   nothing
//
// SIDE EFFECTS:
//===============================================================================
void FLASH_Restore(unsigned char index)
{
unsigned short i;
unsigned short n=0;	//number of last used area in n FLASH sector (is calculated)

//seek the last used area in FLASH sector, returns n -- number of data area
for(i=31;i!=0;i--) if(*(unsigned short*)(((unsigned)index-1)*0x00008000+i*ARRAYLENGHT+FLASH_BASE+ARRAYLENGHT-2)==index) {n=i; break;};

//draw the plot
if(index!=0) DrawPlot(0,(unsigned char*)(((unsigned)index-1)*0x00008000+n*ARRAYLENGHT+FLASH_BASE),(unsigned char*)(((unsigned)index-1)*0x00008000+n*ARRAYLENGHT+FLASH_BASE)+(DATALENGHT-2));
	    else DrawPlot(0,(unsigned char*)DataArray, (unsigned char*)DataArray+(DATALENGHT-2));
ShiftMask=0x0001;

//show Vertical Scale  -- do not in this version
//GUI_PrintBigInteger((unsigned int)VideoArray+(48+XMAX*50), 500, 3, 0);

//show Horizontal Scale			-8 (lenght) -6 (VScale) -4 (HScale) -2 (# of view or "index") 
if(index!=0) i=*(unsigned char*)(((unsigned)index-1)*0x00008000+n*ARRAYLENGHT+FLASH_BASE+ARRAYLENGHT-4);//load saved index of HScale
	    else i=HScale_index;    //or load current index
if (i>10) i=10;

GUI_PrintBigInteger((unsigned int)VideoArray+(48+XMAX*140), 0, 3, 255);
GUI_PrintBigInteger((unsigned int)VideoArray+((6-2*NewHScale_dim1[i])+48+XMAX*140), NewHScale_value[i], NewHScale_dim1[i], NewHScale_dim2[i]);

}

//===============================================================================
// FUNCTION NAME: Algorithm1()
//
// DESCRIPTION:
//            Visualizes all data for high scan rates 0.1-0.2-0.5-1 [mS/div]
//
// ARGUMENTS: none
//
// RETURNS:   nothing
//
// SIDE EFFECTS:
//===============================================================================
void Algorithm1(void)
{
if((AD0CR&0x200000)==0) {//if collection is finished
					     if(Menu==0) {//start again only when Menu=0
					                  DrawPlot(0,(unsigned char*)DataArray,(unsigned char *)cntD[1]);//draw graph
							   	      Level&=0xFF;	//clear	flags to start with triggering
					                     cntD[6]=255-GetVoltages_ShiftLevel+1; //set horizontal ofset
								         cntD[2]=0xFF; cntD[3]=0; cntD[4]=cntD[5];//set init values to start
									     cntD[0]=(unsigned int)DataArray; //start collection again
								      AD0CR|=0x00200000;//Start ADC0 again
								     }//if Menu
                        }
				   else if (cntD[0]==(unsigned int)DataArray) Level|=0x80000000; //if triggering does not occure, set flag to start
}
//===============================================================================
// FUNCTION NAME: Algorithm2()
//
// DESCRIPTION:
//            Visualizes data for middle scan rates 2-5-10-20 [mS/div] (with triggering and hor. ofset)
//
// ARGUMENTS: none
//
// RETURNS:   nothing
//
// SIDE EFFECTS:
//===============================================================================
void Algorithm2(void)
{
if(cntD_old<=(cntD_cur-2)) {DrawPlot(1, (unsigned char *)cntD_old,(unsigned char *)(cntD_cur-2)); //draw graph
                            cntD_old=cntD_cur;
					       }
if((AD0CR&0x200000)==0) {//if collection is finished
                         DrawPlot(1, (unsigned char *)cntD_old,(unsigned char *)cntD[0]); //finish graph drawing
						 if(Menu==0) {//start again only when Menu=0
								      cntD_old=(unsigned int)DataArray;
								      DrawPlot(0, (unsigned char *)cntD_old+1,(unsigned char *)cntD_old); //start drawing graph
                                      Level&=0xFF;	//clear	flags to start with triggering
					                  cntWait=0;	//clear	wait counter
								         cntD[6]=255-GetVoltages_ShiftLevel+1; //set horizontal ofset
                                         cntD[2]=0xFF; cntD[3]=0; cntD[4]=cntD[5];//set init values to start
									     cntD[0]=(unsigned int)DataArray; //start collection again
								      AD0CR|=0x00200000;//Start ADC0 again
								     }//if Menu
                        }
				   else if (cntD_cur==(unsigned int)DataArray) {cntWait++;
				                                                if(cntWait>NewHScale_value[HScale_index]){cntWait=255; Level|=0x80000000;} //if triggering does not occure, set flag to start
															   }
// 20 mS/div x 20 = 400mS = 20 times of 20 mS (50Hz)      
}
//===============================================================================
// FUNCTION NAME: Algorithm3()
//
// DESCRIPTION:
//            Visualizes data for low scan rates 50-100-200 [mS/div] (without triggering and hor. ofset)
//
// ARGUMENTS: none
//
// RETURNS:   nothing
//
// SIDE EFFECTS:
//===============================================================================
void Algorithm3(void)
{
if(cntD_old<=(cntD_cur-2)) {DrawPlot(1, (unsigned char *)cntD_old,(unsigned char *)(cntD_cur-2)); //draw graph
                            cntD_old=cntD_cur;
					       }
if((AD0CR&0x200000)==0) {//if collection is finished
                         DrawPlot(1, (unsigned char *)cntD_old,(unsigned char *)cntD[0]); //finish graph drawing
					     if(Menu==0) {//start again only when Menu=0
								      cntD_old=(unsigned int)DataArray;
								      DrawPlot(0, (unsigned char *)cntD_old+1,(unsigned char *)cntD_old); //start drawing graph
                                      Level&=0xFF;	//clear	flags to start with triggering
					                  Level|=0x80000000; //set flag to start without triggering
								        cntD[6]=1; //set horizontal ofset=0 forewer
                                        cntD[2]=0xFF; cntD[3]=0; cntD[4]=cntD[5];//set init values to start
									    cntD[0]=(unsigned int)DataArray; //start collection again
								      AD0CR|=0x00200000;//Start ADC0 again
								     }//if Menu
                        }
				   else Level|=0x80000000; //set flag to start without triggering (always)
}
//===============================================================================
// FUNCTION NAME: Algorithm4()
//
// DESCRIPTION:
//            Visualizes data for superlow scan rates 0.5-1-2-5-10... [S/div], roll mode
//
// ARGUMENTS: none
//
// RETURNS:   nothing
//
// SIDE EFFECTS:
//===============================================================================
void Algorithm4(void)
{
if(cntD[0]>=cntD[1]) {//if collection is finished
					  if(Menu==0) {//start again only when Menu=0
								   for(ptrD=(unsigned short *)DataArray;ptrD<(unsigned short *)(cntD[1]-2);ptrD++) *ptrD=*(ptrD+1);//shift left data in array
								   GUI_ShiftPlotArea();
								   ptrD=(unsigned short *)cntD[1]-1;
								   //first=0;
								   mask=2; ofs=19; DrawPlot(1, (unsigned char *)ptrD, (unsigned char *)ptrD); //draw graph

								   cntD[0]=cntD[1]-2;

								   Level|=0x80000000; //set flag to start without triggering
								   cntD[6]=1; //set horizontal ofset=0 forewer
                                   if((AD0CR&0x200000)==0) {cntD[2]=0xFF; cntD[3]=0; cntD[4]=cntD[5]; AD0CR|=0x00200000;}//Start ADC0 again
								  }//if Menu
                     }
}
//===============================================================================
// FUNCTION NAME: Visualization()
//
// DESCRIPTION:
//            Visualizes the data using a specific algorithm (depending from mS/div)
//
// ARGUMENTS: none
//
// RETURNS:   nothing
//
// SIDE EFFECTS:
//===============================================================================
void Visualization(void)
{
//	0   1   2  3    4  5  6  7     8   9  10           11 12 13 14 15...
//0.1-0.2-0.5--1    2--5-10-20    50-100-200  mS/div   500-1--2--5-10... S/div
cntD_cur=cntD[0];//fix current pointer

if(HScale_index<4) Algorithm1(); 
                   else if(HScale_index<8) Algorithm2(); 
                                           else if(HScale_index<11) Algorithm3(); else Algorithm4(); 
}

//===============================================================================



