//======================================================================================================
// FILENAME: VideoController.c
//
// DESCRIPTION: This is a file when the video controller is implemented
//
// NOTE: Next routine and pointer can be used:
//       VideoController_Start() -- starts the video contoller
//		 VideoArray[]    		 -- array of video data (frame buffer)
//
//       Also set for specific application:
//		 USED_INTERRUPTS_MASK 	 -- mask for interrupts that are used and must be enabled in application
//
// (c) Entry_AR1755, 2005
//     Philips 2005 ARM Design Contest
//======================================================================================================
  #define USED_INTERRUPTS_MASK 0x00040000	 //the ADC0 interrupt is used and must be enabled in application 
//======================================================================================================
#include <LPC213x.H> 
#define XMAX  64       //bytes per line	 (horizontalal resolution)
#define YMAX 240       //total visible lines (vertical resolution)

unsigned int cntN=0;                     //SSP FIFO cycles counter
unsigned short *cntA=0;                  //Current Address for regeneration

unsigned int VideoArray[(XMAX*YMAX/4)];//Video Array
unsigned short cntL[9]; //multipurpose structure: TV line counter + array of parameters for PAL/NTSC mode

//=============================== PWM Interrupt ==================================
void FIQ_Handler (void) __fiq {
__asm {
  LDR         R11,=0xFFFFF000	  //i/o base address
//what source: SSP or PWM?
  LDR         R10,[R11,#0x04]//VICFIQStatus
  TST         R10,#0x800
  BEQ         pwm_ //is not from SSP, go to PWM's case
//-------------------------- from SSP ----------------------------
//SSPDR=*cntA++; SSPDR=*cntA++; SSPDR=*cntA++; SSPDR=*cntA++;
  LDR         R10,=0xE0068008//SSPDR
  LDR         R8,=cntA
  LDR         R9,[R8]    //<-cntA
  LDRH        R8,[R9],#2 //cntA++
  STRH        R8,[R10]
  LDRH        R8,[R9],#2 //cntA++
  STRH        R8,[R10]
  LDRH        R8,[R9],#2 //cntA++
  STRH        R8,[R10]
  LDRH        R8,[R9],#2 //cntA++
  STRH        R8,[R10]
  LDR         R8,=cntA
  STR         R9,[R8]    //->cntA

  //cntN++;
  LDR         R8,=cntN  // cntN
  LDR         R9,[R8]   // cntN
  ADD         R9,R9,#1
  STR         R9,[R8]   // cntN
  //if (cntN>4) SSPIMSC=0x0;        //Disable  SSP interrupt, the current TV line is over
  CMP         R9,#5
  BLS         no_dis
  MOV         R9,#0
  LDR         R8,=0xE0068014 //SSPIMSC
  STR         R9,[R8]
no_dis:
  B           handler_end



//---------------- now interrupt is from PWM  --------------------
pwm_:
//VICIntEnClr = 0x00040000;        //disable ADC Interrupt
  MOV         R9,#0xFFFFF6FF	   //disable ALL unwanted interrupts
  STR         R9,[R11,#0x14]
//PWMIR       = 0x401;            //Clear interrupt flag
  LDR         R9,=0x401
  LDR         R10,=0xE0014000
  STR         R9,[R10]
//PCON=1; //go to idle mode, wait for other interrupt (is expected from PWM0)
  MOV         R9,#1
  LDR         R10,=0xE01FC0C0
  STRB        R9,[R10]
//VICIntEnable = 0x00040000;      //Enable ADC Interrupt
  MOV         R9,#USED_INTERRUPTS_MASK	//Enable interrupts
  STR         R9,[R11,#0x10]

//-------- now the next PWM interrupt resumes execution -----------

  LDR         R10,=cntL   //base address of cntL[]
  LDRH        R8,[R10]    //load cntL for compare
//if (cntL[0]>=cntL[4])
  LDRH        R9,[R10,#8] //XL3 (end line+1 of the picture)
  CMP         R8,R9
  BCC         a_1		 //if XL3< cntL
//{if (cntL[0]>=cntL[1])
  LDRH        R9,[R10,#2] //XL0 (total lines)
  CMP         R8,R9
  BCC         a_2		 //XL0< cntL
// {cntL[0]=0;  PWMMR4=XP4; PWMMR5=XP5; PWMLER=0x30;   //to line 0, normal PWM
  MOV         R8,#0
  STRH        R8,[R10]

  LDRH        R11,[R10,#14]	//XP4
  LDR         R9,=0xE0014040//PWMMR4
  STR         R11,[R9]

  LDRH        R11,[R10,#16]	//XP5
  LDR         R9,=0xE0014044//PWMMR5
  STR         R11,[R9]

  MOV         R11,#0x30
  LDR         R9,=0xE0014050//PWMLER
  STR         R11,[R9]
//cntA=VideoArray;//point again to beginning of video array  
  LDR         R11,=VideoArray
  LDR         R9,=cntA
  STR         R11,[R9]
//}
  B           a_5
  a_2:
//else if (cntL[0]==cntL[5])
  LDRH        R9,[R10,#10]  //XL0-XL1 (begining of horiz. sync)
  CMP         R8,R9
  BNE         a_5		   //(XL0-XL1)!= cntL
// {PWMMR4=XP5; PWMMR5=XP4; PWMLER=0x30;}  //invert PWM (for V sync pulse)
  LDRH        R11,[R10,#16]	//XP5
  LDR         R9,=0xE0014040//PWMMR4
  STR         R11,[R9]

  LDRH        R11,[R10,#14]	//XP4
  LDR         R9,=0xE0014044//PWMMR5
  STR         R11,[R9]

  MOV         R11,#0x30
  LDR         R9,=0xE0014050//PWMLER
  STR         R11,[R9]
//}
  B           a_5
  a_1:
//if (cntL[0]>=cntL[3])
  LDRH        R9,[R10,#6]   //XL2 (line when the picture starts)
  CMP         R8,R9
  BCC         a_5		  //XL2< cntL
       //fill the FIFO
       //SSPDR=*cntA++; SSPDR=*cntA++; SSPDR=*cntA++; SSPDR=*cntA++; SSPDR=*cntA++;
       //SSPDR=*cntA++; SSPDR=*cntA++; SSPDR=*cntA++; SSPDR=*cntA++; SSPDR=*cntA++;
       LDR         R10,=0xE0068008 //SSPDR
       LDR         R11,=cntA
       LDR         R9,[R11]    //<-cntA
       LDRH        R11,[R9],#2 //cntA ++
       STRH        R11,[R10]
       LDRH        R11,[R9],#2 //cntA ++
       STRH        R11,[R10]
       LDRH        R11,[R9],#2 //cntA ++
       STRH        R11,[R10]
       LDRH        R11,[R9],#2 //cntA ++
       STRH        R11,[R10]
       LDRH        R11,[R9],#2 //cntA ++
       STRH        R11,[R10]
       LDRH        R11,[R9],#2 //cntA ++
       STRH        R11,[R10]
       LDRH        R11,[R9],#2 //cntA ++
       STRH        R11,[R10]
       LDRH        R11,[R9],#2 //cntA ++
       STRH        R11,[R10]
       LDR         R11,=cntA
       STR         R9,[R11]    //->cntA
//SSPIMSC=0x8;          // Enable SPI1 Interrupt when Tx FIFO is 1/2 empty.
  MOV         R11,#0x8
  LDR         R9,=0xE0068014//SSPIMSC
  STR         R11,[R9]
//cntN=0;
  MOV         R11,#0
  LDR         R9,=cntN
  STR         R11,[R9]

  a_5:
//cntL[0]++;            //increment line counter
  LDR         R10,=cntL
  ADD         R8,R8,#1
  STRH        R8,[R10] //-> cntL

//PWMIR       = 0x401;  //Clear interrupt flags
  LDR         R9,=0x401
  LDR         R8,=0xE0014000//PWMIR
  STR         R9,[R8]
handler_end:
  }//asm
}
//===============================================================================
// FUNCTION NAME: VideoController_Start(index)
//
// DESCRIPTION:
//            Starts video controller in PAL or NTSC mode
//
// ARGUMENTS: index:
//            1 -- PAL mode,
//            2 -- NTSC mode
//
// RETURNS:   Nothing
// SIDE EFFECTS:
//===============================================================================
void VideoController_Start (char index)  {
if (index==2) {//set NTSC mode
               cntL[0]=1;//line counter
			   cntL[1]=262;			   //total lines in frame
			   cntL[2]=3;			   //lines per VSync pulse
			   cntL[3]=15;			   //first visible line (line when the picture starts)
			   cntL[4]=cntL[3]+YMAX;   //end line+1 of the picture
               cntL[5]=cntL[1]-cntL[2];//line when Vert.Sync is raised
			   cntL[6]=3812;		   //total PWM ticks for 64 uS line
			   cntL[7]=3517;		   //tick when HSync is raised
			   cntL[8]=cntL[7]-282;	   //tick when HSync is falled
			  }
        else  {//set PAL mode
		       cntL[0]=1;//line counter
			   cntL[1]=312;			   //total lines in frame
			   cntL[2]=3;			   //lines per VSync pulse
			   cntL[3]=50;			   //first visible line (line when the picture starts)
			   cntL[4]=cntL[3]+YMAX;   //end line+1 of the picture
               cntL[5]=cntL[1]-cntL[2];//line when Vert.Sync is raised
			   cntL[6]=3840;		   //total PWM ticks for 64 uS line
			   cntL[7]=3540;		   //tick when HSync is raised
			   cntL[8]=cntL[7]-282;	   //tick when HSync is falled
			  };//if

//---------------- set the GPIO ----------------
PINSEL1|=0x480; //function: P0.21 - PWM5, P0.19 - MOSI1)
VPBDIV=1;       //pclk=cclk/1=60MHz

//---------------- set the PWM ----------------
PWMPR=0;        //Prescale Register: pclk/1
//PWMPC=0;      //Prescale Counter Register
PWMMR0=cntL[6];     //Match Control Register 0
PWMMR4=cntL[7];     //Match Control Register 4
PWMMR5=cntL[8];     //Match Control Register 5
PWMMR6=cntL[6]-100; //Match Control Register 6	(  ) 200  150...120	  70...75

PWMMCR=0x40003; //Match CR (interrupt/reset/stop): Reset on PWMMR0, interrupts PWMMR 0,6 are on
PWMPCR=0x2020;  //CR: enables  PWM5 output, selects double edge mode for the PWM5,
                //PWM6 is single edged and don't have own output
PWMTCR=0x9;     //CR: PWM mode is enabled, Prescale Counter are enabled

//---------------- set the SSP ----------------
SSPCR0= 0x21F;      //CR0: Clock rate=60MHz/2/(2+1), fhase/polarity, SSI format, 16 bits
SSPCR1= 0x2;        //CR1: SSP Enable, Master mode
SSPCPSR=0x2;        //Clock Prescale Register: 2 is min value
//Note: Enabling/disabling of SPI1 Interrupt (when Tx FIFO is 1/2 empty) is located in PWM0/SSP ISR bodies
//------------ set the interrupts in VIC -------------
VICIntSelect |= 0x00000100;			//Make PWM0 Interrupt as FIQ, with "FIQ_Handler()"
VICIntEnable = 0x00000100;          //Enable PWM0 Interrupt

VICIntSelect |= 0x00000800;			//Make SPI Interrupt as FIQ, with "FIQ_Handler()"
VICIntEnable = 0x00000800;          //Enable SPI1 Interrupt

}
