//===============================================================================
// FILENAME: VideoGUI.c
//
// DESCRIPTION: This file contains drawing routines
//
// NOTE: 
// 
// 
// (c) Entry_AR1755, 2005 
//     Philips 2005 ARM Design Contest
//===============================================================================

#define XMAX  64       //bytes per line	 (horizontalal resolution)
#define YMAX 240       //total visible lines (vertical resolution)

#define CHARS 256       //total chars in font
#define CMAX   10       //bytes per char (height in pixels)

#define OFSET 639       //ofset= XMAX*CMAX-1
extern unsigned int VideoArray[];


//===============================================================================
// FUNCTION NAME: GUI_DrawRect()
//
// DESCRIPTION:
//            Draws rectangle in fixed position
//
// ARGUMENTS: None
//
// RETURNS:	  Nothing
// SIDE EFFECTS:
//===============================================================================
void GUI_DrawRect(void)  {
unsigned int i;
unsigned int j;
unsigned int * AAA=0; //Address for GUI

//draw the border of rectangle 
AAA=VideoArray+161+128+32;//point to corner
for (i=0; i<10; i++) {*(AAA+i)|=0xFFFFFFFF; *(AAA+(8*20+1)*64/4+i)|=0xFFFFFFFF;}	//draw horizontal lines
for (i=0; i<(8*20+2); i++) {*(AAA+16*i-1)|=0x000030000;//draw  left vertical line
                            *(AAA+16*i+10)|=0xC000;		//draw  right vertical line
				           }//for i
}


//===============================================================================
// FUNCTION NAME: GUI_ClrScr()
//
// DESCRIPTION:
//            Clears the screen
//
// ARGUMENTS: None
//
// RETURNS:	  Nothing
// SIDE EFFECTS:
//===============================================================================
void GUI_ClrScr(void) __arm 
{
__asm{
  MOV R0,#0 //pattern
  LDR R1,=VideoArray
  LDR R2,=3840 //!!XMAX/4*YMAX
loop0:
  STR R0,[R1],#4
  SUBS R2,R2,#1
  BNE loop0
  }//asm
}

//===============================================================================
// FUNCTION NAME: GUI_DrawWallpaper()
//
// DESCRIPTION:
//            Draws the wallpaper
//
// ARGUMENTS: None
//
// RETURNS:	  Nothing
// SIDE EFFECTS: 
//===============================================================================
  const unsigned short Element1[10]={0x5555, 0xAAAA, 0x5555, 0xAAAA, 0x5555, 0xAAAA, 0x5555, 0xAAAA, 0x5555, 0xAAAA};
  const unsigned short Element2[10]={0xFFFE, 0xAAAA, 0xD554, 0xAAAA, 0xD554, 0xAAAA, 0xD554, 0xAAAA, 0xD554, 0x8000};
//===============================================================================
void GUI_DrawWallpaper(void) __arm 
{
__asm{
  LDR R4,=VideoArray
  MOV R2,#24 //!!YMAX/CMAX
loop2:
  MOV R1,#10
  LDR R3,=Element1
loop1:
  MOV R0,#32 //!!XMAX/2
  LDRH R5,[R3],#2
loop0:
  STRH R5,[R4],#2
  SUBS R0,R0,#1
  BNE loop0

  SUBS R1,R1,#1
  BNE loop1

  SUBS R2,R2,#1
  BNE loop2
  }//asm
}
/*
//===============================================================================
// FUNCTION NAME: GUI_DrawGrid10x8()
//
// DESCRIPTION:
//            Clears the area and draws the grid 10x8
//
// ARGUMENTS: None
//
// RETURNS:	  Nothing
// SIDE EFFECTS: 
//===============================================================================
  const unsigned int Element10x8[20]={0,0x8000,0,0x8000,0,0x8000,0,0x8000,0,0x8000,0,0x8000,0,0x8000,0,0x8000,0,0x8000,0,0xAAAAAAAA};
//===============================================================================
void GUI_DrawGrid10x8(void) __arm 
{
__asm{
  LDR R4,=VideoArray
  LDR R3,=1348 //left upper coner
  ADD R4,R4,R3
  MOV R2,#8 //number of vertical divs
loop2:
  MOV R1,#20  //lines per div
  LDR R3,=Element10x8
loop1:
  MOV R0,#10 //width in 32-bit words
  LDR R5,[R3],#4 //new pattern
loop0:
  STR R5,[R4],#4
  SUBS R0,R0,#1
  BNE loop0

  ADD R4,R4,#24 //next addr = addr + (XMAX/4-width-1)

  SUBS R1,R1,#1
  BNE loop1

  SUBS R2,R2,#1
  BNE loop2
  }//asm
}

//===============================================================================
// FUNCTION NAME: GUI_DrawGrid()
//
// DESCRIPTION:
//            Clears the area and draws the grid 10x6.6
//
// ARGUMENTS: None
//
// RETURNS:	  Nothing
// SIDE EFFECTS: 
//===============================================================================
  const unsigned int Element0[24]={0,0x8000,0,0x8000,0,0x8000,0,0xAAAAAAAA,0,0x8000,0,0x8000,0,0x8000,0,0x8000,0,0x8000,0,0x8000,0,0x8000,0,0x8000};
//===============================================================================
void GUI_DrawGrid(void) __arm 
{
__asm{
  LDR R4,=VideoArray
  LDR R3,=1348 //left upper coner
  ADD R4,R4,R3
  MOV R2,#160 //total lines
loop2:
  MOV R1,#24  //lines per div
  LDR R3,=Element0
loop1:
  MOV R0,#10 //width in 32-bit words
  LDR R5,[R3],#4 //new pattern
loop0:
  STR R5,[R4],#4
  SUBS R0,R0,#1
  BNE loop0

  ADD R4,R4,#24 //next addr = addr + (XMAX/4-width-1)

  SUBS R2,R2,#1
  BEQ end
  SUBS R1,R1,#1
  BNE loop1
  B   loop2
end:
  }//asm
}
*/


//===============================================================================
// FUNCTION NAME: GUI_DrawGrid_1(col)
//
// DESCRIPTION:
//            Clears vertical area 16x160 and draws  grid 1 x 6.6 divisions
//
// ARGUMENTS: None
//
// RETURNS:	  Nothing
// SIDE EFFECTS: R0 is modified
//===============================================================================
  const unsigned short Element[24]={0x8000,0,0x8000,0,0x8000,0,0x8000,0,0x8000,0,0x8000,0,0x8000,0,0x8000,0,0xAAAA,0,0x8000,0,0x8000,0,0x8000,0};   //for bottom-ajusted grid
  //const unsigned short Element[24]={0,0x8000,0,0x8000,0,0x8000,0,0xAAAA,0,0x8000,0,0x8000,0,0x8000,0,0x8000,0,0x8000,0,0x8000,0,0x8000,0,0x8000};	 //for centered grid
//===============================================================================
void GUI_DrawGrid_1(unsigned int col) __arm 
{//R0 -- col
//clear vert column
__asm{
  LDR R2,=VideoArray
  MOV R0,R0,LSL#1
  ADD R2,R2,R0
  LDR R0,=1348 //left upper coner
  ADD R2,R2,R0

  MOV R1,#160 //total lines
loop2:
  MOV R0,#24  //lines per div
  LDR R3,=Element
loop1:
  LDRH R4,[R3],#2 //new pattern
  STRH R4,[R2],#XMAX //next addr = addr+XMAX

  SUBS R1,R1,#1
  BEQ end

  SUBS R0,R0,#1
  BNE loop1
  B   loop2
end:
  }//asm

}

//===============================================================================
// FUNCTION NAME: GUI_ShiftPlotArea()
//
// DESCRIPTION:
//            Shifts left plot area and draws new fragment of grid
//
// ARGUMENTS: None
//
// RETURNS:   Nothing
// SIDE EFFECTS:
//===============================================================================
  unsigned short ShiftMask=0x0001;
//===============================================================================
void GUI_ShiftPlotArea(void) __arm 
{
__asm{
      //rotate ShiftMask>>1
	  LDR  R4,=ShiftMask //load old mask value
      LDRH R0,[R4]
      MOVS  R0,R0,LSR#1	//rotate right 16-bit value
      MOVEQ R0,#0x8000	//rotate right 16-bit value
      STRH R0,[R4]      //store new mask value

  LDR R5,=VideoArray
  LDR R0,=1348 //left upper coner
  ADD R5,R5,R0
  MOV R0,#38   //point to end addr in line
  ADD R5,R5,R0

  MOV R1,#160 //total lines

loop2:
  MOV R2,#24  //lines per div
  LDR R3,=Element

loop1:
  LDRH R4,[R3],#2   //load pattern
  LDR  R0,=ShiftMask //load mask
  LDRH R0,[R0]
  TST  R4,R0        //test bit using mask
  MOVNE R4,#0x10000 //set "C" if pattern bit=1

  MOV  R0,#20       //counter=width of plot area (in 16-bit words)

loop0:
  MOVS R4,R4,LSL#16 //previous "C"
  LDRH R4,[R5]      //
  ADC  R4,R4,R4     //shift left with "C"
  STRH R4,[R5],#-2  //next addr
SUBS R0,R0,#1
BNE loop0

  SUBS R1,R1,#1
  BEQ end

  ADD R5,R5,#104 //next addr = addr + (XMAX+width) = addr + 64+40
SUBS R2,R2,#1
BNE loop1
B   loop2
end:
  }//asm

}



//===============================================================================
// FUNCTION NAME: GUI_DrawPicture (addr, ptr)
//
// DESCRIPTION:
//			  Draws a W x H bitmap at the specified address in video array
//
// ARGUMENTS: None
//            addr -- absolute address (must be even)
//            ptr -- pointer to picture data   [short Width(in 2-bytes), short Height(in pixels), shorts data0, data2, ...]
//
// RETURNS:	  Nothing
// SIDE EFFECTS:
//===============================================================================
void GUI_DrawPicture(unsigned int addr, unsigned int ptr) __arm 
{//R0=addr, R1=ptr
__asm{
  LDRH R4,[R1],#2 //load Width
  LDRH R3,[R1],#2 //load Height

loop1:
  MOV R2,R4	  //count
loop0:
  LDRH R5,[R1],#2
  STRH R5,[R0],#2
  SUBS R2,R2,#1
  BNE loop0

  ADD R0,R0,#XMAX	//to next line
  SUB R0,R0,R4,LSL #1

  SUBS R3,R3,#1
  BNE loop1
  }//asm
}


//===============================================================================
// FUNCTION NAME: GUI_PrintChar (addr, ch)
//
// DESCRIPTION:
//            Draws a char at the specified address in video array
//
// ARGUMENTS: addr -- absolute address
//            ch   -- char
//
// RETURNS:   Incremented address
// SIDE EFFECTS: No range control of addr
//===============================================================================
  const char Font1[];
  const char Font2[];
//===============================================================================
unsigned int GUI_PrintChar(unsigned int addr, char ch) __arm 
{//R0=addr, R1=char, returns (in R0) the next address
__asm{

  LDR R2,=Font2
  ADD R1,R1,R2
  EOR  R0,R0,#0x1   //swap the odd and even addresses

  LDRB R2,[R1],#CHARS
  STRB R2,[R0],#XMAX
  LDRB R2,[R1],#CHARS
  STRB R2,[R0],#XMAX
  LDRB R2,[R1],#CHARS
  STRB R2,[R0],#XMAX
  LDRB R2,[R1],#CHARS
  STRB R2,[R0],#XMAX
  LDRB R2,[R1],#CHARS
  STRB R2,[R0],#XMAX
  LDRB R2,[R1],#CHARS
  STRB R2,[R0],#XMAX
  LDRB R2,[R1],#CHARS
  STRB R2,[R0],#XMAX
  LDRB R2,[R1],#CHARS
  STRB R2,[R0],#XMAX
  LDRB R2,[R1],#CHARS
  STRB R2,[R0],#XMAX
  LDRB R2,[R1],#CHARS
  STRB R2,[R0],#XMAX

  EOR  R0,R0,#0x1   //restore the odd/even address
  LDRB R2,[R0,#-OFSET]!   //R0=R0-640+1, next address	   -639
  }//asm
}
//===============================================================================
// FUNCTION NAME: GUI_PrintCharEx (addr, ch)
//
// DESCRIPTION:
//            Draws a char at the specified address in video array using
//            a current mode (normal or invert)
//
// ARGUMENTS: addr -- absolute address
//            ch   -- char
//
// RETURNS:   Incremented address
// SIDE EFFECTS: No range control of addr
//===============================================================================
  char mode=0;	//print mode  0 -- normal, 0xFF -- invert
//===============================================================================
unsigned int GUI_PrintCharEx(unsigned int addr, char ch) __arm 
{//R0=addr, R1=char, returns (in R0) the next address
__asm{

  LDR R2,=Font2
  ADD R1,R1,R2
  EOR  R0,R0,#0x1   //swap odd and even addresses

  LDR R3,=mode
  LDRB R3,[R3]

  LDRB R2,[R1],#CHARS
  EOR  R2,R2,R3
  STRB R2,[R0],#XMAX
  LDRB R2,[R1],#CHARS
  EOR  R2,R2,R3
  STRB R2,[R0],#XMAX
  LDRB R2,[R1],#CHARS
  EOR  R2,R2,R3
  STRB R2,[R0],#XMAX
  LDRB R2,[R1],#CHARS
  EOR  R2,R2,R3
  STRB R2,[R0],#XMAX
  LDRB R2,[R1],#CHARS
  EOR  R2,R2,R3
  STRB R2,[R0],#XMAX
  LDRB R2,[R1],#CHARS
  EOR  R2,R2,R3
  STRB R2,[R0],#XMAX
  LDRB R2,[R1],#CHARS
  EOR  R2,R2,R3
  STRB R2,[R0],#XMAX
  LDRB R2,[R1],#CHARS
  EOR  R2,R2,R3
  STRB R2,[R0],#XMAX
  LDRB R2,[R1],#CHARS
  EOR  R2,R2,R3
  STRB R2,[R0],#XMAX
  LDRB R2,[R1],#CHARS
  EOR  R2,R2,R3
  STRB R2,[R0],#XMAX

  EOR  R0,R0,#0x1   //restore  odd/even address
  LDRB R2,[R0,#-OFSET]!   //R0=R0-640+1, next address	   -639
  }//asm
}


//===============================================================================
// FUNCTION NAME: GUI_PrintBigDigit(addr, dig)
//
// DESCRIPTION:
//            Draws a stylized digit 16x20 at the specified address in video array
//
// ARGUMENTS: addr -- absolute address
//            dig   -- digit	0...9,   0xA as " "    and   0xB as "-"
//
// RETURNS:   Address incremented by 2
// SIDE EFFECTS: No range control of addr, addr must be even
//===============================================================================
  const short FontDig[];
//===============================================================================
unsigned int GUI_PrintBigDigit(unsigned int addr, char dig) __arm 
{//R0=addr, dig=digit, returns (in R0) the next address
__asm{

  ADD R1,R1,R1
  LDR R2,=FontDig
  ADD R1,R1,R2
  MOV R3,#20
loop1:
  LDRH R2,[R1],#24 //total chars in font
  STRH R2,[R0],#XMAX
  SUBS R3,R3,#1
  BNE loop1

  LDRB R2,[R0],#-1278 //next address
  }//asm
}

//===============================================================================
// FUNCTION NAME: GUI_PrintBigInteger(addr, val, total, point)
//
// DESCRIPTION:
//            Draws an integer as N digits 16x20 at the specified address in video array
//
// ARGUMENTS: addr -- absolute address
//            val  -- unsigned value 0...
//			  total	-- number of digits
//			  point	-- position of decimal point:  255 -- clear all position,
//                                                 0 -- no point,
//												   1 -- xxx.x, 2 -- xx.xx, 3 -- x.xxx
//
// RETURNS:   Address incremented by value=number of digits * 2
// SIDE EFFECTS: No range control of all parameters, addr must be even
//===============================================================================
unsigned int GUI_PrintBigInteger(unsigned int addr, unsigned int val, char total, unsigned char point)
{
char i;
unsigned int digit;
unsigned int power[9]={1,1,10,100,1000,10000,100000,1000000,10000000};
for (i=total;i>0;i--)
	{digit=val/power[i];	 //extract new digit
                 if(point==255) digit=10;	//case of blank position
	 addr=GUI_PrintBigDigit(addr, (char)digit);
	 val=val-digit*power[i];
	}//for i
//show decimal point
if ((point!=0)&(point!=255)) {*((short*)addr+17*XMAX/2-point-1)|=3;
                              *((short*)addr+18*XMAX/2-point-1)|=3;
			                 };
return(addr);
}




//===============================================================================
// FUNCTION NAME: GUI_PrintStrFast (Row, X, string)
//
// DESCRIPTION:
//            Draws a text string at the position specified by X and Y
//
// ARGUMENTS: Row  -- vertical position (row) 0...23
//            X    -- horizontal position (column) 0...63
//			  StrP -- text string
//
// RETURNS:   Nothing
// SIDE EFFECTS: If position X will be >63 the string will be printed in the next
//            pixel line(s). No range control of X, Y and lenght of string 
//===============================================================================
void GUI_PrintStrFast(unsigned int Row, unsigned int X, char*  StrP)
{
char a=*StrP++;
X=X+Row*XMAX*CMAX+(unsigned int)VideoArray; //calculate abs addr
while (a!=0) {X=GUI_PrintChar(X,a); a=*StrP++;};
}



//===============================================================================
// FUNCTION NAME: GUI_PrintStr (Row, X, string)
//
// DESCRIPTION:
//            Draws a text string at the position specified by X and Y
//            The next control code is allowed:
//			      0    ("\x00")     -- end of string
//			      1,XX ("\x01\xXX") -- repeat XX times the next symbol
//			      2,XX ("\x02\xXX") -- skip XX positions
//			      3,XX ("\x03\xXX") -- skip XX lines (micro step down)
//			      4    ("\x04")     -- turn off the Invert Mode
//			      5    ("\x05")     -- turn on the Invert Mode
//			      A    ("\n")       -- to beginning of next row
//
// ARGUMENTS: Row  -- vertical position (row) 0...23
//            X    -- horizontal position (column) 0...63
//			  StrP -- text string
//
// RETURNS:   Nothing
// SIDE EFFECTS: If the screen will be full the next symbol will be printed
//               in the left top corner
//===============================================================================
void GUI_PrintStr(unsigned int Row, unsigned int X, char*  StrP)
{
int n,i;
unsigned int max=XMAX*(YMAX-CMAX+1)+(unsigned int)VideoArray;  //for compare
char a=*StrP++;
unsigned int AA=X+Row*XMAX*CMAX+(unsigned int)VideoArray; //calculate abs addr			       

while (a!=0) {if (a>0xF) {if(AA>=max) {AA=(unsigned int)VideoArray;	X=0;}
                          AA=GUI_PrintCharEx(AA,a);
						  X++; if(X>=XMAX) {X=0; AA=AA+(CMAX-1)*XMAX;}
						 }
					 else{switch (a){
							//repeat
							case 1: {n=*StrP++; a=*StrP++; for(i=0; i<n; i++) {if(AA>=max) {AA=(unsigned int)VideoArray; X=0;}
							                                                   AA=GUI_PrintCharEx(AA,a);
																			   X++; if(X>=XMAX) {X=0; AA=AA+(CMAX-1)*XMAX;}
																			  }//forY
									 break;}
							//skip positions
							case 2: {n=*StrP++;	 for(i=0; i<n; i++) {if(AA>=max) {AA=(unsigned int)VideoArray; X=0;}
							                                         AA++; X++; if(X>=XMAX) {X=0; AA=AA+(CMAX-1)*XMAX;}
																    }//forY
									 break;}
							//skip lines
							case 3: {n=*StrP++; AA=AA+n*XMAX; if(AA>=max) {AA=(unsigned int)VideoArray; X=0;} break;}
							//Normal Mode
							case 4: {mode=0;    break;}
							//Invert Mode
							case 5: {mode=0xFF; break;}
							//to beginning of next row
							case 0xA: {AA=AA+CMAX*XMAX-X; if(AA>=max) AA=(unsigned int)VideoArray; X=0; break;}
							};//switch
						 };//if
			  a=*StrP++;
             };//while
}

//===============================================================================
// FUNCTION NAME: GUI_Invert(Row, X, microstep, Width, Height)
//
// DESCRIPTION:
//            Inverts a piece of string at the position specified by X and Y
//
// ARGUMENTS: Row  -- vertical position (row) 0...23
//            X    -- horizontal position (column) 0...63
//			  microstep  -- vertical ofset 
//			  Width -- width of piece (bytes)
//            Height -- height (pixels)
//
// RETURNS:   Nothing
// SIDE EFFECTS: 
//===============================================================================
void GUI_Invert(unsigned int Row, unsigned int X, char microstep, char Width, char Height) 
{
int i,j;
unsigned int AA=X+(Row*CMAX+microstep)*XMAX+(unsigned int)VideoArray; //calculate abs addr			       
for (i=0;i<Height;i++)
for (j=0;j<Width; j++) *(  (char*)((AA+j+i*XMAX)^1)  )^=0xFF; //invert a byte in video area
}
//===============================================================================
// FUNCTION NAME: GUI_RestoreWallpaper(Row, X, microstep, Width, Height)
//
// DESCRIPTION:
//            Restores a piece of string at the position specified by X and Y as a wallpaper
//
// ARGUMENTS: Row  -- vertical position (row) 0...23
//            X    -- horizontal position (column) 0...63
//			  microstep  -- vertical ofset 
//			  Width -- width of piece (bytes)
//            Height -- height (pixels)
//
// RETURNS:   Nothing
// SIDE EFFECTS: 
//===============================================================================
void GUI_RestoreWallpaper(unsigned int Row, unsigned int X, char microstep, char Width, char Height)
{
int i,j;
unsigned int AA=X+(Row*CMAX+microstep)*XMAX+(unsigned int)VideoArray; //calculate abs addr			       
for (i=0;i<Height;i++)
for (j=0;j<Width; j++) *(  (char*)((AA+j+i*XMAX)^1)  )=*(  (char*)((unsigned int)Element1+(j&1)+i*2) ); //restore byte
}

//===============================================================================
const char Font1[] = {
//Font 8x10, Windows Codepage - 1251
0x00
};
//===============================================================================
const char Font2[] = {
//Font 8x10 bold, Windows Codepage - 1251
//0
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,
0xFC,0xFE,0x00,0x38,0x00,0x00,0x30,0x30,0x3E,0x40,0x7C,0x00,0xD8,0xE6,0xFC,0xEE,0x60,0x70,0x38,0xD8,0x6C,0x00,0x00,0x00,0xFE,0xFB,0x00,0x00,0x00,0x18,0x40,0x00,0x00,0xEE,0x38,0x3C,0x00,0x06,0x00,0x7C,0xFC,0x38,0x7E,0x00,0x00,0x00,0x3C,0xFC,0x38,0x30,0xFC,0x30,0x04,0x00,0x7E,0x00,0x48,0x9E,0x00,0x00,0x30,0x74,0x00,0x48,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
//1
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x30,0x6C,0x6C,0x30,0x00,0x38,0x30,0x18,0x60,0x00,0x00,0x00,0x00,0x00,0x06,0x78,0x30,0x78,0x78,0x1C,0xFC,0x38,0xFC,0x78,0x78,0x00,0x00,0x18,0x00,0x60,0x78,
0x7C,0x30,0xFC,0x3C,0xFC,0xFE,0xFE,0x3C,0xCC,0x78,0x1E,0xE6,0xF0,0xC6,0xC6,0x38,0xFC,0x78,0xFC,0x78,0xFC,0xCC,0xCC,0xC6,0xC6,0xCC,0xFE,0x78,0xC0,0x78,0x10,0x00,0x30,0x00,0xE0,0x00,0x1C,0x00,0x38,0x00,0xE0,0x30,0x18,0xE0,0x70,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x10,0x00,0x00,0x00,0x00,0x00,0x00,0x1C,0x18,0xE0,0x76,0x11,
0x24,0x66,0x00,0x00,0x00,0x00,0x30,0x30,0x62,0xA0,0x28,0x00,0xD8,0x6E,0x64,0xCC,0xF8,0x30,0x30,0x48,0x48,0x00,0x00,0x00,0xC6,0x5B,0x00,0x00,0x00,0x00,0xF0,0x00,0x00,0x46,0x00,0x18,0x84,0xFC,0x30,0x44,0x44,0x44,0xCE,0x00,0x00,0x00,0x42,0x30,0x6C,0x30,0x30,0x00,0xFC,0x00,0xB4,0x00,0x00,0x90,0x00,0x00,0x00,0xCC,0x00,0x00,
0x3E,0xFE,0xFC,0xFE,0x1E,0xFE,0xD6,0x78,0xC6,0xBA,0xE6,0x1E,0xC6,0xC6,0x38,0xFE,0xFC,0x3C,0xFC,0xC6,0x7C,0xC6,0xCC,0xC6,0xD6,0xD6,0xF0,0xC6,0x60,0x78,0xDC,0x7E,0x00,0x06,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x18,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
//2
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x78,0x6C,0x6C,0x7C,0xC6,0x6C,0x30,0x30,0x30,0x6C,0x30,0x00,0x00,0x00,0x0C,0xCC,0xF0,0xCC,0xCC,0x3C,0xC0,0x60,0xCC,0xCC,0xCC,0x30,0x30,0x30,0x00,0x30,0xCC,
0xC6,0x78,0x66,0x66,0x6C,0x62,0x62,0x66,0xCC,0x30,0x0C,0x66,0x60,0xEE,0xE6,0x6C,0x66,0xCC,0x66,0xCC,0xB4,0xCC,0xCC,0xC6,0xC6,0xCC,0xCC,0x60,0x60,0x18,0x38,0x00,0x30,0x00,0x60,0x00,0x0C,0x00,0x6C,0x00,0x60,0x00,0x00,0x60,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x30,0x18,0x30,0xDC,0x38,
0x20,0x66,0x00,0xFE,0x00,0x00,0xFC,0xFC,0xFC,0x5C,0x28,0x20,0xD8,0x78,0x60,0xCC,0x60,0x18,0x60,0x24,0x90,0x78,0x00,0x00,0xC6,0x55,0xF8,0x10,0xEC,0xEC,0x40,0xEE,0x00,0x2C,0xCE,0x18,0x78,0x60,0x30,0xE0,0x50,0x9A,0xC6,0x00,0x00,0x00,0xBD,0x30,0x6C,0x30,0x30,0x70,0x60,0xC8,0xB4,0x00,0x78,0xD0,0x7E,0x00,0x78,0xC0,0x7C,0x78,
0x66,0x66,0x66,0x66,0x36,0x62,0xD6,0xCC,0xC6,0xC6,0x66,0x76,0xEE,0xC6,0x6C,0xC6,0x66,0x66,0xB4,0xC6,0xD6,0xC6,0xCC,0xC6,0xD6,0xD6,0xB0,0xC6,0x60,0xCC,0xD6,0xC6,0x00,0x7C,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x10,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x10,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
//3
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x78,0x6C,0xFE,0xC0,0xCC,0x38,0x60,0x60,0x18,0x38,0x30,0x00,0x00,0x00,0x18,0xDC,0x30,0x0C,0x0C,0x6C,0xF8,0xC0,0x0C,0xCC,0xCC,0x30,0x30,0x60,0xFC,0x18,0x0C,
0xDE,0xCC,0x66,0xC0,0x66,0x68,0x68,0xC0,0xCC,0x30,0x0C,0x6C,0x60,0xFE,0xF6,0xC6,0x66,0xCC,0x66,0xE0,0x30,0xCC,0xCC,0xC6,0x6C,0xCC,0x98,0x60,0x30,0x18,0x6C,0x00,0x18,0x78,0x7C,0x78,0x0C,0x78,0x60,0x76,0x6C,0x70,0x78,0x66,0x30,0xEC,0xF8,0x78,0xDC,0x76,0xD8,0x7C,0x7C,0xCC,0xCC,0xC6,0xC6,0xCC,0xFC,0x30,0x18,0x30,0x00,0x6C,
0x2C,0x60,0x00,0x66,0x00,0x00,0x30,0x30,0x60,0x70,0x2C,0x40,0xD8,0x70,0x6C,0xCC,0x7C,0x08,0x40,0x00,0x00,0x78,0x00,0x00,0xC6,0xFB,0x50,0x08,0x48,0x54,0x78,0x44,0x00,0x2C,0x86,0x18,0x48,0x60,0x30,0x90,0x70,0xA2,0xF0,0x24,0x00,0x00,0x95,0x30,0x38,0xFC,0x30,0x30,0x60,0xCC,0xB4,0x30,0xCC,0xD6,0xC6,0x90,0x18,0x78,0xC4,0x18,
0xC6,0x60,0x66,0x60,0x66,0x68,0x7C,0x0C,0xCE,0xCE,0x6C,0x66,0xFE,0xC6,0xC6,0xC6,0x66,0xC0,0x30,0xC6,0xD6,0x6C,0xCC,0xC6,0xD6,0xD6,0x30,0xC6,0x60,0x06,0xD6,0xC6,0x78,0xC0,0xFC,0xFE,0x3C,0x78,0xD6,0x7C,0xC6,0xC6,0xE6,0x3E,0xC6,0xC6,0x78,0xFE,0xFC,0x78,0xFC,0xC6,0x7C,0xC6,0xCC,0xC6,0xD6,0xD6,0xF0,0xC6,0x60,0x7C,0xDC,0x7E,
//4
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x30,0x00,0x6C,0x78,0x18,0x76,0x00,0x60,0x18,0xFE,0xFC,0x00,0xFC,0x00,0x30,0xFC,0x30,0x38,0x38,0xCC,0x0C,0xF8,0x18,0x78,0x7C,0x00,0x00,0xC0,0x00,0x0C,0x18,
0xD6,0xCC,0x7C,0xC0,0x66,0x78,0x78,0xC0,0xFC,0x30,0x0C,0x78,0x60,0xD6,0xDE,0xC6,0x7C,0xCC,0x7C,0x38,0x30,0xCC,0xCC,0xD6,0x38,0x78,0x30,0x60,0x18,0x18,0xC6,0x00,0x00,0x0C,0x66,0xCC,0x7C,0xCC,0xF0,0xCC,0x76,0x30,0x18,0x6C,0x30,0xFE,0xCC,0xCC,0x66,0xCC,0x6C,0xC0,0x30,0xCC,0xCC,0xC6,0x6C,0xCC,0x98,0xE0,0x00,0x1C,0x00,0xC6,
0x32,0x60,0x00,0x66,0x00,0x00,0x30,0x30,0xFC,0x00,0x2A,0x80,0xFC,0x78,0x76,0xCC,0x66,0x00,0x00,0x00,0x00,0x78,0xFC,0xFF,0xC6,0x00,0x5C,0x04,0x7E,0x70,0x44,0x44,0x00,0x18,0x4C,0xD8,0x48,0x60,0x00,0xCC,0x50,0xA2,0xC0,0x48,0xFE,0xFC,0x99,0x30,0x00,0x30,0x30,0x30,0x60,0xCC,0x74,0x30,0xFC,0xB6,0xF0,0x48,0x18,0x0C,0x78,0x18,
0xC6,0x7C,0x7C,0x60,0x66,0x78,0x38,0x38,0xDE,0xDE,0x78,0x66,0xD6,0xFE,0xC6,0xC6,0x7C,0xC0,0x30,0x7E,0xD6,0x38,0xCC,0x7E,0xD6,0xD6,0x3C,0xF2,0x7C,0x3E,0xF6,0x7E,0x0C,0x7C,0x66,0x66,0x6C,0xCC,0x7C,0xC6,0xCE,0xCE,0x6C,0x66,0xEE,0xC6,0xCC,0xC6,0x66,0xCC,0xB4,0xC6,0xD6,0x6C,0xCC,0xC6,0xD6,0xD6,0xB0,0xC6,0x60,0xC6,0xD6,0xC6,
//5
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x30,0x00,0xFE,0x0C,0x30,0xDC,0x00,0x60,0x18,0x38,0x30,0x00,0x00,0x00,0x60,0xEC,0x30,0x60,0x0C,0xFE,0x0C,0xCC,0x30,0xCC,0x0C,0x00,0x00,0x60,0xFC,0x18,0x30,
0xDE,0xFC,0x66,0xC0,0x66,0x68,0x68,0xCE,0xCC,0x30,0xCC,0x6C,0x62,0xC6,0xCE,0xC6,0x60,0xCC,0x78,0x1C,0x30,0xCC,0xCC,0xFE,0x6C,0x30,0x62,0x60,0x0C,0x18,0x00,0x00,0x00,0x7C,0x66,0xC0,0xCC,0xFC,0x60,0xCC,0x66,0x30,0x18,0x78,0x30,0xD6,0xCC,0xCC,0x66,0xCC,0x6C,0x78,0x30,0xCC,0xCC,0xD6,0x38,0xCC,0x30,0x30,0x18,0x30,0x00,0xC6,
0x22,0x60,0x00,0x66,0x00,0x00,0x30,0xFC,0x60,0x12,0x2A,0x80,0xDA,0x6C,0x66,0xCC,0x66,0x00,0x00,0x00,0x00,0x78,0x00,0x00,0xC6,0x00,0x52,0x04,0x4A,0x48,0x44,0x44,0x00,0x18,0x48,0xD8,0x78,0x60,0x00,0x24,0x40,0x9A,0xC0,0x90,0x06,0x00,0xBD,0x30,0x00,0x30,0x30,0x30,0x60,0xCC,0x34,0x00,0xC0,0xB6,0xC0,0x24,0x18,0x0C,0x0C,0x18,
0xFE,0x66,0x66,0x60,0x66,0x68,0x7C,0x0C,0xF6,0xF6,0x6C,0x66,0xC6,0xC6,0xC6,0xC6,0x60,0xC0,0x30,0x06,0x7C,0x6C,0xCC,0x06,0xD6,0xD6,0x36,0xDA,0x66,0x06,0xD6,0x3E,0x7C,0xC6,0x7C,0x60,0x6C,0xFC,0x38,0x1C,0xDE,0xDE,0x78,0x66,0xFE,0xFE,0xCC,0xC6,0x66,0xC0,0x30,0xC6,0xD6,0x38,0xCC,0x7E,0xD6,0xD6,0x3C,0xF2,0x7C,0x1E,0xF6,0x7E,
//6
0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x00,0x00,0x00,0x6C,0xF8,0x66,0xCC,0x00,0x30,0x30,0x6C,0x30,0x70,0x00,0x30,0xC0,0xCC,0x30,0xCC,0xCC,0x0C,0xCC,0xCC,0x60,0xCC,0x18,0x30,0x70,0x30,0x00,0x30,0x00,
0xC0,0xCC,0x66,0x66,0x6C,0x62,0x60,0x66,0xCC,0x30,0xCC,0x66,0x66,0xC6,0xC6,0x6C,0x60,0xDC,0x6C,0xCC,0x30,0xCC,0x78,0xEE,0xC6,0x30,0xC6,0x60,0x06,0x18,0x00,0x00,0x00,0xCC,0x66,0xCC,0xCC,0xC0,0x60,0x7C,0x66,0x30,0x18,0x6C,0x30,0xD6,0xCC,0xCC,0x7C,0x7C,0x60,0x0C,0x34,0xCC,0x78,0xFE,0x6C,0x7C,0x64,0x30,0x18,0x30,0x00,0xC6,
0x22,0x60,0x30,0x60,0x6C,0x00,0x30,0x30,0x66,0x2D,0xAA,0x40,0xDA,0x66,0x66,0xCC,0x66,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xC6,0x00,0x52,0x08,0x4A,0x44,0x44,0x44,0x00,0xB0,0x38,0xD8,0x84,0x60,0x30,0x1C,0x44,0x44,0xC6,0x48,0x06,0x00,0x42,0x30,0x00,0x30,0x30,0x30,0x60,0xDC,0x34,0x00,0xC0,0x90,0xC6,0x48,0x18,0xCC,0x8C,0x18,
0xC6,0x66,0x66,0x60,0x66,0x62,0xD6,0xCC,0xE6,0xE6,0x66,0x66,0xC6,0xC6,0x6C,0xC6,0x60,0x66,0x30,0xC6,0x10,0xC6,0xCC,0x06,0xD6,0xD6,0x36,0xDA,0x66,0xCC,0xD6,0x66,0xCC,0xC6,0x66,0x60,0x6C,0xC0,0x7C,0xC6,0xF6,0xF6,0x6C,0x66,0xD6,0xC6,0xCC,0xC6,0x7C,0xCC,0x30,0x7E,0xD6,0x6C,0xCC,0x06,0xD6,0xD6,0x36,0xDA,0x66,0xC6,0xD6,0x66,
//7
0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x00,0x30,0x00,0x6C,0x30,0xC6,0x76,0x00,0x18,0x60,0x00,0x00,0x30,0x00,0x30,0x80,0x78,0xFC,0xFC,0x78,0x0C,0x78,0x78,0x60,0x78,0x70,0x30,0x30,0x18,0x00,0x60,0x30,
0x78,0xCC,0xFC,0x3C,0xFC,0xFE,0xF0,0x3E,0xCC,0x78,0x78,0xE6,0xFE,0xC6,0xC6,0x38,0xF0,0x78,0xE6,0x78,0x78,0x78,0x30,0xC6,0xC6,0x78,0xFE,0x78,0x02,0x78,0x00,0x00,0x00,0x76,0xBC,0x78,0x76,0x78,0xF0,0x0C,0xE6,0x78,0xD8,0xE6,0x78,0xC6,0xCC,0x78,0x60,0x0C,0xF0,0xF8,0x18,0x76,0x30,0x6C,0xC6,0x0C,0xFC,0x1C,0x18,0xE0,0x00,0xFE,
0x74,0xF0,0x30,0xF0,0x48,0xA8,0x30,0x30,0x3C,0x12,0xDC,0x20,0xDC,0xE6,0xF6,0xFE,0xE6,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xFE,0x00,0xBC,0x10,0xEE,0xE6,0xEE,0xFE,0x00,0xF0,0x30,0x70,0x00,0xF0,0x30,0x88,0xFC,0x38,0x7C,0x24,0x06,0x00,0x3C,0xFC,0x00,0xFC,0xFC,0xFC,0xF0,0xEE,0x34,0x00,0x7C,0x96,0x7C,0x90,0x18,0xB8,0xF8,0x7E,
0xC6,0xFC,0xFC,0xF0,0xFF,0xFE,0xD6,0x78,0xC6,0xC6,0xE6,0xC6,0xC6,0xC6,0x38,0xC6,0xF0,0x3C,0x78,0x7C,0x38,0xC6,0xFE,0x06,0xFE,0xFF,0x3C,0xF2,0x7C,0x78,0xDC,0xC6,0x76,0x7C,0xFC,0xF0,0xFE,0x78,0xD6,0x7C,0xE6,0xE6,0xE6,0xE6,0xC6,0xC6,0x78,0xC6,0x60,0x78,0x78,0x06,0x7C,0xC6,0xFE,0x06,0xFE,0xFF,0x3C,0xF2,0x7C,0x7C,0xDC,0xC6,
//8
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x60,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x60,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1C,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xFF,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF8,0x00,0x00,0x70,0x00,0x00,0x00,0x00,0x00,0xF0,0x1E,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF8,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x60,0x00,0x90,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x30,0x06,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x10,0x00,0x00,0xA0,0x00,0x00,0x00,0x30,0xF8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xC0,0x76,0x00,0x00,0x00,0x00,0x00,0x18,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0xC3,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x06,0x00,0x00,0x03,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xC6,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x60,0x00,0x00,0x3C,0x10,0x00,0x06,0x00,0x00,0x03,0x00,0x00,0x00,0x00,0x00,0x00,
//9
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x40,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x30,0x3C,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x10,0x00,0x00,0xE0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xC0,0x00,0x00,0x00,0x00,0x00,0x00,0xF0,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
};
//===============================================================================
const short FontDig[240] = {
//Font 16x20 for digits "0"..."9", "_" and "-"
//
0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
0x0FE0,0x0010,0x0FE0,0x0FE0,0x1010,0x0FE0,0x0FE0,0x0FE0,0x0FE0,0x0FE0,0x0000,0x0000,
0x1FF0,0x0038,0x1FF0,0x1FF0,0x3838,0x1FF0,0x1FF0,0x1FF0,0x1FF0,0x1FF0,0x0000,0x0000,
0x3838,0x0038,0x0038,0x0038,0x3838,0x3800,0x3800,0x0038,0x3838,0x3838,0x0000,0x0000,
0x3838,0x0038,0x0038,0x0038,0x3838,0x3800,0x3800,0x0038,0x3838,0x3838,0x0000,0x0000,
0x3838,0x0038,0x0038,0x0038,0x3838,0x3800,0x3800,0x0038,0x3838,0x3838,0x0000,0x0000,
0x3838,0x0038,0x0038,0x0038,0x3838,0x3800,0x3800,0x0038,0x3838,0x3838,0x0000,0x0000,
0x3838,0x0038,0x0038,0x0038,0x3838,0x3800,0x3800,0x0038,0x3838,0x3838,0x0000,0x0000,
0x3838,0x0038,0x0FF0,0x0FF0,0x1FF8,0x1FE0,0x3FE0,0x0038,0x1FF0,0x1FF8,0x0000,0x1FF0,
0x3838,0x0038,0x1FE0,0x0FF0,0x0FF8,0x0FF0,0x3FF0,0x0038,0x1FF0,0x0FF8,0x0000,0x1FF0,
0x3838,0x0038,0x3800,0x0038,0x0038,0x0038,0x3838,0x0038,0x3838,0x0038,0x0000,0x0000,
0x3838,0x0038,0x3800,0x0038,0x0038,0x0038,0x3838,0x0038,0x3838,0x0038,0x0000,0x0000,
0x3838,0x0038,0x3800,0x0038,0x0038,0x0038,0x3838,0x0038,0x3838,0x0038,0x0000,0x0000,
0x3838,0x0038,0x3800,0x0038,0x0038,0x0038,0x3838,0x0038,0x3838,0x0038,0x0000,0x0000,
0x3838,0x0038,0x3800,0x0038,0x0038,0x0038,0x3838,0x0038,0x3838,0x0038,0x0000,0x0000,
0x1FF0,0x0038,0x1FF0,0x1FF0,0x0038,0x1FF0,0x1FF0,0x0038,0x1FF0,0x1FF0,0x0000,0x0000,
0x0FE0,0x0010,0x0FE0,0x0FE0,0x0010,0x0FE0,0x0FE0,0x0010,0x0FE0,0x0FE0,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000
};
//===============================================================================
/*
const short Image1[2+7*80] = {
//Bitmap image of switch
//
7,80, //width, height
//
0xFFFE,0xFFFE,0xFFFF,0xFFFF,0xFFFE,0xFFFE,0xFFFE,
0xAAAA,0xAAAA,0xAAFF,0xFFFF,0xFEAA,0xAAAA,0xAAAA,
0xD554,0xD554,0xDFFF,0xFFFF,0xFFF4,0xD554,0xD554,
0xAAAA,0xAAAA,0xFFFF,0xFFFF,0xFFFE,0xAAAA,0xAAAA,
0xD554,0xD557,0xFFFF,0xFB9F,0xFFFF,0xD554,0xD554,
0xAAAA,0xAABF,0xFFFF,0xF36F,0xFFFF,0xFAAA,0xAAAA,
0xD554,0xD57F,0xFFFF,0xEB6F,0xFFFF,0xFF54,0xD554,
0xAAAA,0xABFF,0xFFFF,0xFB6F,0xFFFF,0xFFAA,0xAAAA,
0xD554,0xD7F9,0xCFFF,0xFB6F,0xFFF8,0x7FF4,0xD554,
0x8000,0x9FF6,0xB7FF,0xFB6F,0xFFFB,0xFFF8,0x8000,
0xFFFE,0xFFFE,0xB7FF,0xF19F,0xFFF8,0xFFFE,0xFFFE,
0xAAAA,0xFFFD,0xB7FF,0xFFFF,0xFFFF,0x7FFF,0xAAAA,
0xD557,0xFFFB,0xB7FF,0xFFFF,0xFFFF,0x7FFF,0xD554,
0xAAAF,0xFFF7,0xB7FF,0xFFFF,0xFFFB,0x7FFF,0xEAAA,
0xD55F,0xFFF0,0xCFFF,0x0000,0xFFFC,0xFFFF,0xF554,
0xAABF,0xFFFF,0xFFE0,0x0000,0x07FF,0xFFFF,0xFAAA,
0xD57F,0xFFFF,0xFF00,0x0000,0x00FF,0xFFFF,0xFD54,
0xAAFF,0xFFFF,0xF800,0x0000,0x001F,0xFFFF,0xFEAA,
0xD5FF,0xFFFF,0xE000,0x0000,0x0007,0xFFFF,0xFF54,
0x81FF,0xFFFF,0x8000,0x0000,0x0001,0xFFFF,0xFF80,
0xFFE1,0x9FFF,0x0001,0xFFFF,0x8000,0xFFFE,0x7FFE,
0xAFEF,0x6FFC,0x001F,0xFFFF,0xF800,0x3FFD,0xBFEA,
0xD7E3,0x6FF8,0x007F,0xFFFF,0xFE00,0x1FFF,0xBFF4,
0xAFFD,0x6FE0,0x03FF,0xFFFF,0xFFC0,0x07FF,0x7FFA,
0xDFFD,0x6FC0,0x07FF,0xFFFF,0xFFE0,0x03FE,0xFFF4,
0xBFED,0x6F80,0x1FFF,0xFFFF,0xFFF8,0x01FD,0xFFFA,
0xFFF3,0x9F00,0x3FFF,0xFFFF,0xFFFC,0x01FC,0x3FFC,
0xBFFF,0xFE00,0xFFFF,0xFFFF,0xFFFF,0x00FF,0xFFFE,
0xFFFF,0xFE00,0xFFFF,0xFFFF,0xFFFF,0x007F,0xFFFC,
0xFFFF,0xFC01,0xFFFF,0xFFFF,0xFFFF,0x803F,0xFFFE,
0xFFFF,0xF803,0xFFFF,0xFFFF,0xFFFF,0xC03F,0xFFFE,
0xFFFF,0xF807,0xFFFF,0xFFFF,0xFFFF,0xE01F,0xFFFF,
0xFFFF,0xF00F,0xFFFF,0xFEFF,0xFFFF,0xF01F,0xFFFF,
0xFFFF,0xF00F,0xFFFF,0xDFF7,0xFFFF,0xF00F,0xFFFF,
0xFFFF,0xF01F,0xFFFF,0xFEFF,0xFFFF,0xF80F,0xFFFF,
0xFFFF,0xE01F,0xFFFD,0xF7DF,0x7FFF,0xF80F,0xFFFF,
0xFFFF,0xE01F,0xFFFF,0xFEFF,0xFFFF,0xF807,0xFFFF,
0xF739,0xE01F,0xFFFF,0xBBBB,0xFFFF,0xF807,0xFBFF,
0xE6D6,0xE03F,0xFFFF,0xEFEF,0xFFFF,0xFC07,0xF3FF,
0xD6D6,0xE03F,0xFFFF,0xFFFF,0xFFFF,0xFC07,0xEBFF,
0xF6D6,0xE03F,0xFFEE,0xDEF6,0xEFFF,0xFC07,0xFBFF,
0xF6D6,0xE03F,0xFFFF,0xFFFF,0xFFFF,0xFC07,0xFBFF,
0xF6D6,0xE01F,0xFFFF,0xEFEF,0xFFFF,0xF807,0xFBFF,
0xE339,0xE01F,0xFFFF,0xBBBB,0xFFFF,0xF80F,0xF1FF,
0xFFFF,0xE01F,0xFFFF,0xFEFF,0xFFFF,0xF80F,0xFFFF,
0xFFFF,0xF01F,0xFFFD,0xF7DF,0x7FFF,0xF80F,0xFFFF,
0xFFFF,0xF00F,0xFFFF,0xFEFF,0xFFFF,0xF00F,0xFFFF,
0xFFFF,0xF00F,0xFFFF,0xDFF7,0xFFFF,0xF01F,0xFFFF,
0xFFFF,0xF807,0xFFFF,0xFEFF,0xFFFF,0xE01F,0xFFFF,
0xFFFF,0xF803,0xFFFF,0xFFFF,0xFFFF,0xC03F,0xFFFE,
0xFFFF,0xFC01,0xFFFF,0xFFFF,0xFFFF,0x803F,0xFFFE,
0xFFFF,0xFE00,0xFFFF,0xFFFF,0xFFFF,0x007F,0xFFFE,
0xFFFF,0xFE00,0xFFFF,0xFFFF,0xFFFF,0x00FF,0xFFFC,
0xBFFF,0xFF00,0x3FFF,0xFFFF,0xFFFC,0x01F3,0xC3FA,
0xDFCE,0x7380,0x1FFF,0xFFFF,0xFFF8,0x01ED,0xDFFC,
0xBFB5,0xADC0,0x07FF,0xFFFF,0xFFE0,0x03ED,0xC7FA,
0xDFF5,0xADE0,0x03FF,0xFFFF,0xFFC0,0x07ED,0xFBF4,
0xAFED,0xADF8,0x007F,0xFFFF,0xFE00,0x1FED,0xFBEA,
0xD7DD,0xADFC,0x001F,0xFFFF,0xF800,0x3FED,0xCBD4,
0x83BD,0xADFF,0x0001,0xFFFF,0x8000,0xFFF3,0x67C0,
0xFF86,0x73FF,0x8000,0x0000,0x0003,0xFFFF,0x7FFE,
0xAAFF,0xFFFF,0xE000,0x0000,0x000F,0xFFFF,0xFFAA,
0xD57F,0xFFFF,0xFC00,0x0000,0x003F,0xFFFF,0xFF54,
0xAABF,0xFFFF,0xFF80,0x0000,0x01FF,0xFFFF,0xFEAA,
0xD55F,0xFFFF,0xFFF0,0x0000,0x1FFF,0xFFFF,0xFD54,
0xAAAF,0xFFC3,0x39FF,0xC003,0xFFCF,0x9FFF,0xFAAA,
0xD557,0xFFDE,0xD6FF,0xFFFF,0xFFB7,0x6FFF,0xD554,
0xAAAB,0xFFC6,0xD6FF,0xFFFF,0xFFB7,0xEFFF,0xAAAA,
0xD554,0xFFFA,0xD6FF,0xFFFF,0xFFB7,0xDFFF,0xD554,
0x8000,0xBFFA,0xD6FF,0xFFFF,0xFFB7,0xBFFC,0x8000,
0xFFFE,0xFFDA,0xD6FF,0xFFFF,0xFFB7,0x7FFE,0xFFFE,
0xAAAA,0xAFE7,0x39FF,0xFFFF,0xFFCD,0x0FEA,0xAAAA,
0xD554,0xD5FF,0xFFFF,0xFFFF,0xFFFD,0xFFD4,0xD554,
0xAAAA,0xAAFF,0xFFFF,0xFFFF,0xFFFF,0xFEAA,0xAAAA,
0xD554,0xD55F,0xFFFF,0xFFFF,0xFFFF,0xFD54,0xD554,
0xAAAA,0xAAAB,0xFFFF,0xFFFF,0xFFFF,0xEAAA,0xAAAA,
0xD554,0xD554,0xFFFF,0xFFFF,0xFFFE,0xD554,0xD554,
0xAAAA,0xAAAA,0xAFFF,0xFFFF,0xFFEA,0xAAAA,0xAAAA,
0xD554,0xD554,0xD57F,0xFFFF,0xFF54,0xD554,0xD554,
0x8000,0x8000,0x8000,0xFFFE,0x8000,0x8000,0x8000
};

//===============================================================================
// FUNCTION NAME: GUI_PrintSwitch
//
// DESCRIPTION:
//            Draws a stylized image of switch (7*16 x 80) at fixed position
//
// ARGUMENTS: none
//
// RETURNS:
// SIDE EFFECTS:
//===============================================================================
void GUI_PrintSwitch(void) 
{
GUI_DrawPicture((unsigned int)VideoArray+(46+60*64), (unsigned int)Image1);
GUI_PrintStr(4,49, "\5\3\5 mS/div \4");
}
*/
//===============================================================================
// FUNCTION NAME: GUI_DrawPixel(X, Y)
//
// DESCRIPTION:
//            Draw a pixel using inverted method
//
// ARGUMENTS: X -- coordinate
//			  Y -- coordinate
//
// RETURNS:   Nothing
// SIDE EFFECTS: 
//===============================================================================
void GUI_DrawPixel(unsigned int X, unsigned int Y) __arm
{//R0-X, R1-Y
__asm{
   //calc address
   LDR R2,=VideoArray
   ADD R2,R2,R1,LSL#6
   ADD R2,R2,R0,LSR#3
   BIC R2,R2,#1
   //calc mask
   MOV   R1,#0x8000
   ANDS  R0,R0,#0xF
   MOVNE R1,R1,LSR R0

   LDRH R0,[R2]
   EOR  R0,R0,R1 //xor mode 
   STRH R0,[R2]
   }//asm
}

//===============================================================================
// FUNCTION NAME: GUI_DrawLine(X1, Y1, X2, Y2)
//
// DESCRIPTION:
//            Draw a pixel using inverted method
//
// ARGUMENTS: X1, Y1, X2, Y2 -- pair of coordinates
//
//
// RETURNS:   Nothing
// SIDE EFFECTS: 
//===============================================================================
void GUI_DrawLine(unsigned int X, unsigned int Y, unsigned int X2, unsigned int Y2) __arm
{//R0-X, R1-Y, R2-X2, R3-Y2 
//work variables:           R4-delta, R5-Addr, R6-mask, R7-counter, R8- +/-XMAX
__asm{
   CMP R2,R0
   MOVLO R8,R3	  //if X2 < X then swap pairs
   MOVLO R3,R1
   MOVLO R1,R8
   MOVLO R8,R2
   MOVLO R2,R0
   MOVLO R0,R8
   SUB R2,R2,R0  //dX (always is positive)
   ADD R2,R2,#1

   MOV R8,#XMAX
   SUBS R3,R3,R1 //dY
   RSBLO R3,R3,#0//if negative then -dY
   RSBLO R8,R8,#0//if negative then -XMAX
   ADD R3,R3,#1

    //calc address
   LDR R5,=VideoArray
   ADD R5,R5,R1,LSL#6
   ADD R5,R5,R0,LSR#3
   BIC R5,R5,#1
   //calc mask
   MOV   R6,#0x8000
   ANDS  R0,R0,#0xF
   MOVNE R6,R6,LSR R0

   MOV R4,#0
   CMP R2,R3  //dX-dY
   BHS alg1	  //if dX >= dY	 then Algorithm 1
   //else Algorithm 2
   MOV R7,R3
loop2:
   //draw point
   LDRH R0,[R5]
   EOR  R0,R0,R6 //xor mode 
   STRH R0,[R5]

   ADD R4,R4,R2
   CMP R4,R3
   SUBHS R4,R4,R3
   BLO skip
	     MOVS R6,R6,LSR#1 //increment X
         MOVEQ R6,#0x8000
         ADDEQ R5,R5,#2
   skip:
   ADD R5,R5,R8          //increment/decrement Y

   SUBS R7,R7,#1
   BNE loop2
   B end

alg1:
   MOV R7,R2
loop1:
   //draw point
   LDRH R0,[R5]
   EOR  R0,R0,R6 //xor mode 
   STRH R0,[R5]

   ADD R4,R4,R3
   CMP R4,R2
   SUBHS R4,R4,R2
   ADDHS R5,R5,R8       //increment/decrement Y

   MOVS R6,R6,LSR#1	    //increment X
   MOVEQ R6,#0x8000
   ADDEQ R5,R5,#2

   SUBS R7,R7,#1
   BNE loop1
end:
   }//asm
}


//===============================================================================
